from typing import List

from pydantic import BaseModel, Field

from ..values import ClipRangeInt
from .image import Resolution


class Mask(BaseModel):
    source: str = Field('',
                        title='Mask file',
                        description='Represent the local path of the mask file')

    output: str = Field('',
                        title='Output file path',
                        description='Represent mask file guid on google drive')

    border: int = Field(40,
                        title='Mask Border width',
                        description='border could help get better result when '
                        'meeting the situation of cutting mask on image edges',
                        ge=0)

    reverse: bool = Field(True,
                          title='Reverse mask',
                          description='Reverse the float value of the grayscale '
                          'mask. (e.g. 1.0 - 0.2 = 0.8)')

    reverse_bias: int = Field(1,
                              title='Bias value to reverse',
                              description='The new grayscale value would be the '
                              'Subtraction of bias and orgin value',
                              ge=1)


    resolution: Resolution = Field(Resolution(width=1024, height=1024),
                                   title='Mask Resolution',
                                   description='Represent the new resolution of mask image')

    clip_range: ClipRangeInt = Field(ClipRangeInt(start=0, end=100),
                                     title='Shadow Shape Scale',
                                     description='Represent the scale value to generate'
                                     'the shadow from DC texture. 0: means no shadow')

    class Config:

        # extra ： https://pydantic-docs.helpmanual.io/usage/model_config
        # 'ignore' will silently ignore any extra attributes,
        # 'allow' will assign the attributes to the model.
        extra = 'allow'



from pydantic import BaseModel, Field, validator

SUPPORT_RESOLUTION = [2048, 1280, 1024, 512, 256, 128, 64, 32, 16]


class Resolution(BaseModel):
    width: int = Field(1024,
                       title='Image Width')

    height: int = Field(1024,
                        title='Image Height')

    @validator('width', 'height')
    def is_valid_resolution_values(cls, v):  # pylint: disable=no-self-argument
        if not v in SUPPORT_RESOLUTION:
            raise ValueError('resolution must be one of the value from '
                             f'the lists \n[{"/".join(SUPPORT_RESOLUTION)}]')
        # if not math.log(v, 2).is_integer():
        #     raise ValueError('must be a power of two value.')
        return v

    @property
    def values(self):
        return (self.width, self.height)



import json
import re
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, validator, Extra


class Metadata(BaseModel, extra=Extra.forbid):
    guid: str = Field(
        "",
        title='Config GUID',
        description='Represent the GUID value'
    )

    publish: int = Field(
        1,
        title='Publishing state',
        description='Represent the index value of the publishing state.'
        'There are four different state level to control the config visiblity '
        'for different dev route: \n'
        '0-> Not publish (e.g. config tempalte)\n'
        '1-> Publish to the internel developing stage\n'
        '2-> Publish to the BETA testing stage\n'
        '3 ~ 9 -> Placeholders for the new requirements of other stages\n'
        '10-> Publish to the Live game',
        ge=0,
        le=10
    )

    parent: Optional[str] = Field(
        '',
        title='Config Parent Path',
        description='Represent the parent path, '
        'it would be used to perform a full copy of '
        'the object payload. Some of the settings are '
        'inherited from parent, the reset of settings are '
        'overrided by itself.'
    )

    tags: Optional[List[str]] = Field(
        [],
        title='Object Tags Information',
        description='Represent the object categories',
        min_items=1
    )

    name: Optional[str] = Field(
        '',
        title='Object Name',
        description="It would be updated to the config file name automatically."
        "It's a read-only property."
    )

    @validator('guid')
    def guid_is_not_empty(cls, v):  # pylint: disable=no-self-argument
        if not v:
            raise ValueError('must contain a valid guid')
        return v

    @validator('parent')
    def parent_path_start_with_root(cls, v):  # pylint: disable=no-self-argument
        if v:
            if not v.startswith('Data'):
                raise ValueError('Invalid path: must starts with "Data" root path')
            if re.fullmatch(r'.*\.\w+', v):
                raise ValueError('Invalid path: can\'t contain file extension')
        return v



from pydantic import BaseModel, Field


class vectorFloat3d(BaseModel):
    x: float = Field(0,
                     title='X - Axis')

    y: float = Field(0,
                     title='Y - Axis')

    z: float = Field(0,
                     title='Z - Axis')

    @property
    def values(self):
        return (self.x, self.y, self.z)

    @property
    def IsZero(self):
        return self.x + self.y + self.z == 0

    class Config:
        schema_extra = {
            'examples': [
                {
                    'x': 2,
                    'y': 0,
                    'z': 1
                }
            ]
        }


class vectorInt3d(BaseModel):
    x: int = Field(0,
                   title='X - Axis')

    y: int = Field(0,
                   title='Y - Axis')

    z: int = Field(0,
                   title='Z - Axis')

    @property
    def is_valid(self):
        return any(self.values)

    @property
    def values(self):
        return (self.x, self.y, self.z)

    class Config:
        schema_extra = {
            'examples': [
                {
                    'x': 2,
                    'y': 0,
                    'z': 1
                }
            ]
        }




from pydantic import BaseModel, Field


class BaseArrayItem(BaseModel):

    name: str = Field(...,
                      title='Item identity',
                      description='Represent the unique item key name')

    is_enabled: bool = Field(True,
                             title='Enable flag',
                             description='Represent the available states.')



import hashlib
import json
import logging
import os
import re
from collections import OrderedDict
from contextlib import contextmanager
from typing import Dict, Tuple

from compipe.exception.validate_error import GErrorValue
from compipe.utils.access import GITHUB_TOKEN_KEY, AccessHub
from compipe.utils.logging import logger
from compipe.utils.parameters import (ARG_DATA, ARG_DIR, ARG_FILE, ARG_OUTPUT,
                                      ARG_PARENT)
from github import Github

DEFAULT_BASE_URL = "https://api.github.com"


@contextmanager
def logger_blocker(logger_level=logging.DEBUG):
    try:
        logger = logging.getLogger()  # pylint: disable=no-member
        ori_level = logger.level
        logger.setLevel(logger_level)
        yield
    finally:
        logger.setLevel(ori_level)


class GithubHelper():

    def __init__(self, repo_name, re_filter_source=None, re_filter_export=None, output=None, base_url=None):

        token = AccessHub().get_credential(GITHUB_TOKEN_KEY)

        self.g = Github(base_url=base_url or DEFAULT_BASE_URL, login_or_token=token)

        # parse repo client
        self.repo_name = repo_name

        with logger_blocker(logger_level=logging.ERROR):  # hide massive http logs
            self.repo = self.g.get_repo(self.repo_name)
        # initialize filters
        self.re_filter_source = re_filter_source
        # keep exsiting file sha information
        self.re_filter_export = re_filter_export or r'^(Export\/Data\/.*)\.json'
        # it could help keep the 'sha' info for the existing file
        self.sha_mappings = {}
        # keep the export config path
        self.output = output

    def get_config(self, name):
        with logger_blocker(logger_level=logging.ERROR):
            return self.repo.get_contents(name)

    def get_properties(self):
        with logger_blocker(logger_level=logging.ERROR):
            file_lists = []
            if not self.repo_name or not self.repo:
                raise GErrorValue('Repo\'s not been initialized.')
            contents = self.repo.get_contents("")
            while contents:
                file_content = contents.pop(0)
                if file_content.type == ARG_DIR:
                    contents.extend(self.repo.get_contents(file_content.path))
                elif not self.re_filter_source or re.fullmatch(self.re_filter_source, file_content.path):
                    file_lists.append(file_content)
                # elif not self.re_filter_export or re.fullmatch(self.re_filter_export, file_content.path):
                    # keep the the 'sha' info to the existing files, it would help to perform 'update'
                    # changes on github
                self.sha_mappings.update({
                    file_content.path: file_content.sha
                })

        logger.debug(f'Loaded properties from repo: [{self.repo_name}]')
        return file_lists


class JsonPropertiesHelper(GithubHelper):

    def get_properties(self):

        configs = super().get_properties()

        properties = {}

        with logger_blocker(logger_level=logging.ERROR):

            for config in configs:

                path, _ = os.path.splitext(config.path)

                json_content = json.loads(config.decoded_content)

                properties.update({
                    path: {
                        ARG_FILE: config.path,
                        ARG_OUTPUT: f'{self.output}/{os.path.basename(config.path)}',
                        ARG_PARENT: json_content.get(ARG_PARENT),
                        ARG_DATA: json_content}
                })

        return properties

    def commit(self, output: str, data: Dict, sha=None, branch='master') -> Tuple[bool, str]:

        # remove temporary (internal) keys from the config data before committing
        _data = OrderedDict(sorted(data.items()))

        # tmp attribute name has '_' prefix
        _tmp_keys = [key for key in _data.keys() if key[0] == '_']

        # remove keys
        for _key in _tmp_keys:

            del _data[_key]

        config_data = json.dumps(_data, indent=4)

        # represent the git status
        # true: add/change
        # message: logging message
        results = (True, 'None')

        with logger_blocker(logger_level=logging.WARNING):

            message_header = '[AUTO] Generate configs'
            # message = f'{message_header} - source:{self.file_path} target:{self.output}'
            # if 'sha' is valid, it would perform update behaviors
            # update_file and create_file methods are the build-in functions
            # from pygithub Repository https://github.com/PyGithub/PyGithub
            if sha:
                # compare the str hash between local data and remote file. It would skip commit if the
                # hash values are the same.
                remote_file_data = json.dumps(json.loads(self.repo.get_contents(output).decoded_content),
                                              indent=4)
                # remote config file hash
                remote_data_hash = hashlib.md5(remote_file_data.encode('utf-8')).hexdigest()

                # local config file hash
                local_data_hash = hashlib.md5(config_data.encode('utf-8')).hexdigest()

                # compare config hash and perform the commit if they were different
                if remote_data_hash != local_data_hash:

                    self.repo.update_file(output,
                                          f'{message_header} [UPDATED]',
                                          config_data,
                                          sha,
                                          branch=branch)

                    results = (True, f'Updated config: {output}')

                else:

                    results = (False, f'[Skip Commit] No change happend on file: [{output}]')

            else:

                self.repo.create_file(output,
                                      f'{message_header} [ADDED]',
                                      config_data,
                                      branch=branch)

                results = (True, f'Added config: {output}')

        return results



"""Populate the property objects
"""

import typing
from typing import List

from pydantic import BaseModel, Field

from .base_asset_wrapper import BaseAssetWrapper


class SchemaRepo(BaseModel):

    repo: str = Field('',
                      title='git repo name',
                      description='represent the repo full path.')

    path: str = Field('',
                      title='the repo path of the schema file',
                      description='represent the resolver name.')

    main_branch: str = Field('',
                             title='main branch name',
                             description='represent the resolver name.')


class ResolverParam(BaseModel):
    #
    filter_source: str = Field('',
                               title='Source filter (regex)',
                               description='represent the source filter pattern for looking for the matched configs')

    filter_export: str = Field('',
                               title='export filter (regex)',
                               description='represent the export fitler pattern which would be used to check if the config'
                               'is an "add" commit. The repo helper would perform "add" behavior instead of "update"')

    output_path: str = Field('',
                             title='Output path',
                             description='the target path for storing the full payload configs.')

    ignore_paths: List[str] = Field([],
                                    title='Ignore path list',
                                    description='represent the paths for excluding the configs when populating full payload data')

    repo: str = Field('',
                      title='git repo name',
                      description='represent the repo full path.')

    base_url: str = Field('',
                          title='Github Enterprise with custom hostname')

    name: str = Field('',
                      title='resolver name',
                      description='represent the resolver name.')

    main_branch: str = Field('',
                             title='main branch name',
                             description='represent the resolver name.')

    model_name: str = Field('',
                            title='model class name',
                            description='represent the pydantic model class name.')

    model: typing.Type[BaseAssetWrapper] = Field(None,
                                                 title='model object',
                                                 description='represent the pydantic model object.')

    resolver: typing.Any = Field(None,
                                 title='resolver object',
                                 description='represent the pydantic resolver object.')

    model_schema: SchemaRepo = Field(None,
                                     title='schema git repo definition',
                                     description='Represent the git repo information which would '
                                     'be used to export the schema file.')



from enum import Enum
from pydantic import Field
from ..array_item import BaseArrayItem


class GSize(str, Enum):
    xxxs = 'xxx_small'
    xxs = 'xx_small'
    xs = 'x_small'
    s = 'small'
    m = 'medium'
    l = 'large'
    xl = 'x_large'
    xxl = 'xx_large'
    xxxl = 'xxx_large'


class PickRule(int, Enum):
    random = 1
    consequent = 2


class Segment(BaseArrayItem):
    value: float = Field(0.05,
                         title='Grayscale Value',
                         description='Represent the value of '
                         'contour level. Range (0.0 ~ 1.0',
                         ge=0.05,
                         le=1.0
                         )

    number: int = Field(5,
                        title='Point numbers',
                        description='Represent the point numbers '
                        'to be generated in the valid contour area.',
                        ge=0
                        )

    radius: int = Field(1,
                        title='Point numbers',
                        description='Represent the point numbers '
                        'to be generated in the valid contour area.',
                        ge=1
                        )

    timeout: int = Field(10,
                         title='Timeout',
                         description='Represent the timeout when '
                         'generating points. The process could be '
                         'forced stop when encounter timeout')

    shadow_scale: float = Field(0,
                                title='Shadow Shape Scale',
                                description='Represent the scale value to generate'
                                'the shadow from DC texture. 0: means no shadow',
                                ge=0,
                                le=1)

    resource: str = Field(...,
                          title='Resource folder ID',
                          description='Represent the folder ID of the '
                          'segment resources on google drive')

    pick_rule: PickRule = Field(PickRule.random,
                                title='Resource picking rule',
                                description='Represent the rule of selecting assets from resource. '
                                '1: select asset randomly from the specific folder.'
                                '2: loop asset consequently from the specific folder.')

    class Config:
        schema_extra = {
            'examples': [{
                'name': GSize.s,
                'enable': True,
                'value': 0.08,
                'timeout': 10,
                'number': 10,
                'radius': 10,
                'resource': '1AAozk1mJOVuY7h5zbDXAzpSx749E9E68'
            }]
        }



from setuptools import setup, find_packages

setup(
    name='asseteour',         # How you named your package folder (MyLib)
    version='0.1.11',      # Start with a small number and increase it with every change you make
    license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='A lite derived configuration system',   # Give a short description about your library
    author='esunvoteb',                   # Type in your name
    author_email='esun@voteb.com',      # Type in your E-Mail
    url='https://github.com/ImagineersHub/asseteour',   # Provide either the link to your github or to your website
    # download_url='https://github.com/ImagineersHub/compipe/archive/v_01.tar.gz',    # I explain this later on
    keywords=['python', 'configuration', 'config', 'derived', 'json'],   # Keywords that define your package best
    packages=find_packages(),
    install_requires=[            # I get to this in a second
        'pydantic>=1.9.0',
        'matplotlib>=3.5.1',
        'compipe>=0.1.14',
        'PyGithub>=1.55'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3.7',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



from abc import ABCMeta, abstractmethod
from typing import Dict


class BaseAssetWrapper(metaclass=ABCMeta):

    @classmethod
    @abstractmethod
    def parse_obj(cls, payload: Dict):
        pass

    @classmethod
    @abstractmethod
    def schema_json(cls, indent=4):
        pass

    @property
    @abstractmethod
    def publish(self):
        pass

    @abstractmethod
    def json(self):
        pass

    @abstractmethod
    def yaml(self):
        pass




import glob
import os
import random
from abc import ABCMeta, abstractmethod
from collections import namedtuple
from enum import Enum


class AbstractAssetBuilder(metaclass=ABCMeta):

    @abstractmethod
    def build(self):
        """Build assets with the assigned config.
        """
        pass

    @abstractmethod
    def pre_build(self):
        """Implement the behaviors to prepare the data before creating new assets.
        """
        pass

    @abstractmethod
    def post_build(self):
        """Implement the behaviors to response the build results.
        """
        pass

    @abstractmethod
    def load_cache(self):
        """Implement the behaviors to response the build results.
        """
        pass

    @abstractmethod
    def run(self, cache: bool = False):
        """Build assets with the given skin ID.
        """
        pass

    @abstractmethod
    def validator(self):
        """Invlove the function to validate assets.
        """
        pass

    @abstractmethod
    def export_cache(self):
        pass


class AssetBuilder(AbstractAssetBuilder):
    def __init__(self, config):
        pass

    def validator(self):
        pass

    def build(self):
        pass

    def load_cache(self):
        pass

    def pre_build(self):
        pass

    def post_build(self):
        pass

    def run(self, cache: bool = False):

        self.pre_build()

        self.validator()

        if cache:
            self.load_cache()
        else:
            self.build()

        return self.post_build()

    def export_cache(self):
        pass
